from fugashi import GenericTagger
import ipadic
import textspan
import tokenizers

import os


#
# Pretokenizers for Japanese sequences
#
class IpadicTagger(GenericTagger):
    """
    fugashi with ipadic
    """
    def __init__(self):
        super().__init__(ipadic.MECAB_ARGS)


class IpadicPreTokenizer(object):
    """
    PreTokenizer with IpadicTagger
    Note that, since this PreTokenizer is not serializable,
    we have to load model and pretokenizer separately.
    """

    @classmethod
    def make(cls):
        """instantiate a PreTokenizer object."""
        return tokenizers.pre_tokenizers.PreTokenizer.custom(cls())

    def __init__(self):

        self.tagger = IpadicTagger()

    def _pre_tokenize(self, _id, ns):

        text = ns.normalized
        tokens = [n.surface for n in self.tagger.parseToNodeList(text)]
        tokens_spans = textspan.get_original_spans(tokens, text)
        return [ns[sp:ep] for sub_spans in tokens_spans for sp, ep in sub_spans]

    def pre_tokenize(self, pretok):

        pretok.split(self._pre_tokenize)


class ManbyoDictTagger(GenericTagger):
    """
    fugashi with ipadic and manbyo dictionaries
    """
    MANBYO_DICT_PATH = os.environ["MANBYO_DICT_PATH"]

    MECAB_ARGS = ' '.join([ipadic.MECAB_ARGS, '-u ' + MANBYO_DICT_PATH])

    def __init__(self):
        super().__init__(self.MECAB_ARGS)


class ManbyoDictPreTokenizer(IpadicPreTokenizer):
    """
    PreTokenizer with CustomTagger
    Note that, since this PreTokenizer is not serializable,
    we have to load model and pretokenizer separately.
    """

    def __init__(self):

        self.tagger = ManbyoDictTagger()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['japre']

package_data = \
{'': ['*']}

install_requires = \
['flake8>=4.0.1,<5.0.0',
 'fugashi>=1.1.2,<2.0.0',
 'ipadic>=1.0.0,<2.0.0',
 'pytextspan>=0.5.4,<0.6.0',
 'tokenizers>=0.12.1,<0.13.0']

setup_kwargs = {
    'name': 'japre',
    'version': '0.1.0',
    'description': 'Custom pretokenizers for Japanese language models',
    'long_description': '# japre',
    'author': 'Kaito Sugimoto',
    'author_email': 'kaito_sugimoto@is.s.u-tokyo.ac.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Alab-NII/japre',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
